@@ -28,7 +28,7 @@ use AutoLoader qw(AUTOLOAD);
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw( ); 
 our @EXPORT = qw( ); 
-our $VERSION = sprintf("%.02f",(substr q$Revision: 2.1 $, 10));
+our $VERSION = sprintf("%.02f",(substr q$Revision: 2.3 $, 10));
 
 sub new {
     my $proto = shift;
@@ -105,6 +105,7 @@ sub distance_from {
 	my $dp = $p2->subtract($p1);
 	sqrt( ($dp->X ** 2) + ($dp->Y **2) );
 }
+sub distance_to { distance_from(@_); }
 
 sub angle_to {
 	my($p1,$p2) = @_;
@@ -333,7 +334,7 @@ Geo::ShapeFile
 
 =head1 AUTHOR
 
-Jason Kohles, E<lt>jason@localdomainE<gt>
+Jason Kohles, E<lt>email@jasonkohles.com<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
